﻿		
		var canvas_offset = 4,
			bubble_padding = 15,
			inner_radius_offset = 200,
			stroke_thickness = 4,
			stroke_opacity = 0.3,
			rect_height = 150,
			rect_width = 8,
			top_offset = -30,
			type = "description";
		
		var radius,
			width,
			height,
			svg,
			pack,
			nest,
			link,
			node,
			revenue_data,
			link_data;
					
		$(document).ready(function(){
			"use strict";
						
			d3.json("data/revenue_data.json", function(error, loaded_data) {
				revenue_data = loaded_data;
				
				d3.json("data/cross_selling_potential.json", function(error, raw_data) {
					link_data = raw_data;
					draw(type);
				});
			});
		}); 
		
		function draw(type)
		{
			width = $("#content").width();
			height = $("#content").height();
			
			var diameter = width / 2.0,
				radius = diameter / 2,
				innerRadius = radius - inner_radius_offset;
				
			var cluster = d3.layout.cluster()
						.size([360, innerRadius])
						.sort(comparator)
						.value(function(d) { return d.values; });
						
			var bundle = d3.layout.bundle();

			var line = d3.svg.line.radial()
						.interpolate("bundle")
						.tension(0.85)
						.radius(function(d) { return d.y; })
						.angle(function(d) { return d.x / 180 * Math.PI; });
			
			var transparent_scale = d3.scale.linear().domain([0, d3.max(link_data, function(d){return d.lift})]).range([0, stroke_opacity]);
			var size_scale = d3.scale.linear().domain([0, d3.max(link_data, function(d){return d.lift})]).range([1, stroke_thickness]);
			
			svg = d3.select("#content").append("svg")
					.attr("width", width)
					.attr("height", height)
				  .append("g")
					.attr("transform", "translate(" + (width / 2.0) + "," + (radius + top_offset )+ ")");
					
			link = svg.append("g").selectAll(".link");
			node = svg.append("g").selectAll(".node");
			
			nest = d3.nest().key(function(d) { return d[type]; }).rollup(function(leaves) { return d3.sum(leaves, function(d) { return Math.max(parseFloat(d.revenue), 0.0);})}).entries(revenue_data.children);
			
			revenue_data.children = nest;
			var rect_size_scale = d3.scale.linear().domain([0, d3.max(revenue_data.children, function(d){return parseFloat(d.values);})]).range([0, rect_height]);
			generateLinks();
			
			var nodes = cluster.nodes(revenue_data),
				links = link_data;
			
			link = link.data(bundle(links))
					.enter().append("path")
					  .each(function(d) { d.source = d[0], d.target = d[d.length - 1]; })
					  .attr("class", "link")
					  .style("stroke-opacity", function(d, i){ return transparent_scale(link_data[i].lift); })
					  .style("stroke-width", function(d, i){ return size_scale(link_data[i].lift); })
					  .attr("d", line);
			
			svg.selectAll(".node_rect_shape")
			  .data(nodes.filter(function(n) { return !n.children; }))
			.enter().append("g")
			  .attr("class", "node_rect")
			  .attr("transform", function(d) { return "rotate(" + (d.x - 90) + ")translate(" + (d.y + rect_width)+ ")"; })
			.append("rect")
			  .attr("x", function(d) { return -(rect_width/2.0); })
			  .attr("width", function(d) { return rect_width; })
			  .attr("height", function(d) { return rect_size_scale(parseFloat(d.values)); })			 		 
			  .attr("class", "node_rect_shape")
			  .attr("transform", function(d) { return "rotate(" + (-90) + ")"; })
			  .on("mouseover", mouseovered)
			  .on("mouseout", mouseouted); 
			
			node = node.data(nodes.filter(function(n) { return !n.children; }))
					.enter().append("g")
					  .attr("class", "node")
					  .attr("transform", function(d) { return "rotate(" + (d.x - 90) + ")translate(" + d.y + ")"; })
					.append("text")
					  .attr("dx", function(d) { return d.x < 180 ? rect_width : -rect_width; })
					  .attr("dy", ".31em")
					  .attr("text-anchor", function(d) { return d.x < 180 ? "start" : "end"; })
					  .attr("transform", function(d) { return d.x < 180 ? null : "rotate(180)"; })
					  .text(function(d) { return d.key; })
					  .on("mouseover", mouseovered)
					  .on("mouseout", mouseouted);
		}
		
		function comparator(a, b) {
		  return b.value - a.value;
		}
		
		function generateLinks(){
			
			for(var i= 0; i< link_data.length; i++){
				var link = link_data[i];
				var source = revenue_element(link.source);
				var target = revenue_element(link.target);
				
				link.source = source;
				link.target = target;
			}
		}
		
		function revenue_element(key){
			
			for(var i = 0; revenue_data.children.length; i++){
				var selected = revenue_data.children[i];
				
				if(selected.key == key){
					return selected;					
				}
			}
			
			return "empty";
		}
		
		function mouseovered(d) {
		  node
			  .each(function(n) { n.target = n.source = false; });

		  link
			  .classed("link--target", function(l) { if (l.target === d) return l.source.source = true; })
			  .classed("link--source", function(l) { if (l.source === d) return l.target.target = true; })
			.filter(function(l) { return l.target === d || l.source === d; })
			  .each(function() { this.parentNode.appendChild(this); });		  
		}

		function mouseouted(d) {
		  link
			  .classed("link--target", false)
			  .classed("link--source", false);

		  
		}
		